@@ -121,9 +121,11 @@ document 7
 # TODO: unicode support is pretty sloppy. define it better.
 
 from datetime import datetime
+import htmlentitydefs
 import logging
 import re
 import time
+import types
 import urllib
 import urllib2
 from urlparse import urlsplit, urlunsplit
@@ -157,7 +159,7 @@ try:
     from httplib2 import Http
     TIMEOUTS_AVAILABLE = True
 except ImportError:
-    from httplib import HTTPConnection
+    from httplib import HTTPConnection, HTTPSConnection
     TIMEOUTS_AVAILABLE = False
 
 try:
@@ -165,9 +167,17 @@ try:
 except NameError:
     from sets import Set as set
 
+try:
+    # TODO: perhaps refactor to requests when https://github.com/kennethreitz/requests/issues/68 lands?
+    from poster.encode import multipart_encode
+    POSTER_AVAILABLE = True
+except ImportError:
+    POSTER_AVAILABLE = False
+
+
 __author__ = 'Joseph Kocherhans, Jacob Kaplan-Moss, Daniel Lindsley'
 __all__ = ['Solr']
-__version__ = (2, 0, 14)
+__version__ = (2, 1, 0)
 
 def get_version():
     return "%s.%s.%s" % __version__[:3]
@@ -194,29 +204,57 @@ if False:
     LOG.addHandler(stream)
 
 
+def unescape_html(text):
+    """
+    Removes HTML or XML character references and entities from a text string.
+
+    @param text The HTML (or XML) source text.
+    @return The plain text, as a Unicode string, if necessary.
+
+    Source: http://effbot.org/zone/re-sub.htm#unescape-html
+    """
+    def fixup(m):
+        text = m.group(0)
+        if text[:2] == "&#":
+            # character reference
+            try:
+                if text[:3] == "&#x":
+                    return unichr(int(text[3:-1], 16))
+                else:
+                    return unichr(int(text[2:-1]))
+            except ValueError:
+                pass
+        else:
+            # named entity
+            try:
+                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
+            except KeyError:
+                pass
+        return text # leave as is
+    return re.sub("&#?\w+;", fixup, text)
 
 def safe_urlencode(params, doseq=0):
     """
     UTF-8-safe version of safe_urlencode
-    
+
     The stdlib safe_urlencode prior to Python 3.x chokes on UTF-8 values
     which can't fail down to ascii.
     """
     if hasattr(params, "items"):
         params = params.items()
-    
+
     new_params = list()
-    
+
     for k, v in params:
         k = k.encode("utf-8")
-        
+
         if isinstance(v, basestring):
             new_params.append((k, v.encode("utf-8")))
         elif isinstance(v, (list, tuple)):
             new_params.append((k, [i.encode("utf-8") for i in v]))
         else:
             new_params.append((k, unicode(v)))
-    
+
     return urllib.urlencode(new_params, doseq)
 
 
@@ -225,13 +263,18 @@ class SolrError(Exception):
 
 
 class Results(object):
-    def __init__(self, docs, hits, highlighting=None, facets=None, spellcheck=None, stats=None):
+    def __init__(self, docs, hits, highlighting=None, facets=None,
+                 spellcheck=None, stats=None, qtime=None, debug=None,
+                 grouped=None):
         self.docs = docs
         self.hits = hits
         self.highlighting = highlighting or {}
         self.facets = facets or {}
         self.spellcheck = spellcheck or {}
         self.stats = stats or {}
+        self.qtime = qtime
+        self.debug = debug or {}
+        self.grouped = grouped or {}
 
     def __len__(self):
         return len(self.docs)
@@ -255,57 +298,79 @@ class Solr(object):
         self.path = path.rstrip('/')
         self.timeout = timeout
         self.log = self._get_log()
-    
+
     def _get_log(self):
         return LOG
-    
+
     def _send_request(self, method, path, body=None, headers=None):
         if TIMEOUTS_AVAILABLE:
             http = Http(timeout=self.timeout)
             url = self.base_url + path
-            
+
+            self.log.debug("Starting request to '%s' (%s) with body '%s'...",
+                           url, method, str(body)[:10])
+            start_time = time.time()
+
             try:
-                start_time = time.time()
-                self.log.debug("Starting request to '%s' (%s) with body '%s'..." % (url, method, str(body)[:10]))
                 headers, response = http.request(url, method=method, body=body, headers=headers)
-                end_time = time.time()
-                self.log.info("Finished '%s' (%s) with body '%s' in %0.3f seconds." % (url, method, str(body)[:10], end_time - start_time))
-            except AttributeError:
-                # For httplib2.
-                error_message = "Failed to connect to server at '%s'. Are you sure '%s' is correct? Checking it in a browser might help..." % (url, self.base_url)
-                self.log.error(error_message)
-                raise SolrError(error_message)
-            
+            except (IOError, AttributeError):
+                error_message = "Failed to connect to server at '%s'. Are you sure '%s' is correct? Checking it in a browser might help..."
+                params = (url, self.base_url)
+                self.log.error(error_message, *params, exc_info=True)
+                raise SolrError(error_message % params)
+
+            end_time = time.time()
+            self.log.info("Finished '%s' (%s) with body '%s' in %0.3f seconds.",
+                          url, method, str(body)[:10], end_time - start_time)
+
             if int(headers['status']) != 200:
                 error_message = self._extract_error(headers, response)
-                self.log.error(error_message)
+                self.log.error(error_message, extra={'data': {'headers': headers,
+                                                              'response': response}})
                 raise SolrError(error_message)
-            
+
             return response
         else:
             if headers is None:
                 headers = {}
-            
-            conn = HTTPConnection(self.host, self.port)
+
+            if self.scheme == 'http':
+                conn = HTTPConnection(self.host, self.port)
+            elif self.scheme == 'https':
+                conn = HTTPSConnection(self.host, self.port)
+
             start_time = time.time()
-            self.log.debug("Starting request to '%s:%s/%s' (%s) with body '%s'..." % (self.host, self.port, path, method, str(body)[:10]))
-            conn.request(method, path, body, headers)
+            self.log.debug("Starting request to '%s:%s/%s' (%s) with body '%s'...",
+                           self.host, self.port, path, method, str(body)[:10])
+
+            try:
+                conn.request(method, path, body, headers)
+            except IOError:
+                error_message = "Failed to connect to server at '%s'. Are you sure '%s' is correct? Checking it in a browser might help..."
+                params = (path, self.base_url)
+                self.log.error(error_message, *params, exc_info=True)
+                raise SolrError(error_message % params)
+
             response = conn.getresponse()
             end_time = time.time()
-            self.log.info("Finished '%s:%s/%s' (%s) with body '%s' in %0.3f seconds." % (self.host, self.port, path, method, str(body)[:10], end_time - start_time))
-            
+            self.log.info("Finished '%s:%s/%s' (%s) with body '%s' in %0.3f seconds.",
+                          self.host, self.port, path, method, str(body)[:10], end_time - start_time)
+
             if response.status != 200:
-                error_message = self._extract_error(dict(response.getheaders()), response.read())
-                self.log.error(error_message)
+                resp_headers = dict(response.getheaders())
+                resp_body = response.read()
+                error_message = self._extract_error(resp_headers, resp_body)
+                self.log.error(error_message, extra={'data': {'headers': resp_headers,
+                                                              'response': resp_body}})
                 raise SolrError(error_message)
-            
+
             return response.read()
 
     def _select(self, params):
         # specify json encoding of results
         params['wt'] = 'json'
         params_encoded = safe_urlencode(params, True)
-        
+
         if len(params_encoded) < 1024:
             # Typical case.
             path = '%s/select/?%s' % (self.path, params_encoded)
@@ -317,29 +382,29 @@ class Solr(object):
                 'Content-type': 'application/x-www-form-urlencoded; charset=utf-8',
             }
             return self._send_request('POST', path, body=params_encoded, headers=headers)
-    
+
     def _mlt(self, params):
         params['wt'] = 'json' # specify json encoding of results
         path = '%s/mlt/?%s' % (self.path, safe_urlencode(params, True))
         return self._send_request('GET', path)
-    
+
     def _suggest_terms(self, params):
         params['wt'] = 'json' # specify json encoding of results
         path = '%s/terms/?%s' % (self.path, safe_urlencode(params, True))
         return self._send_request('GET', path)
-    
+
     def _update(self, message, clean_ctrl_chars=True, commit=True, waitFlush=None, waitSearcher=None):
         """
         Posts the given xml message to http://<host>:<port>/solr/update and
         returns the result.
-        
+
         Passing `sanitize` as False will prevent the message from being cleaned
         of control characters (default True). This is done by default because
         these characters would cause Solr to fail to parse the XML. Only pass
         False if you're positive your data is clean.
         """
         path = '%s/update/' % self.path
-        
+
         # Per http://wiki.apache.org/solr/UpdateXmlMessages, we can append a
         # ``commit=true`` to the URL and have the commit happen without a
         # second request.
@@ -352,31 +417,31 @@ class Solr(object):
             query_vars.append('waitSearcher=%s' % str(bool(waitSearcher)).lower())
         if query_vars:
             path = '%s?%s' % (path, '&'.join(query_vars))
-            
-        
+
+
         # Clean the message of ctrl characters.
         if clean_ctrl_chars:
             message = sanitize(message)
-        
+
         return self._send_request('POST', path, message, {'Content-type': 'text/xml; charset=utf-8'})
-    
+
     def _extract_error(self, headers, response):
         """
         Extract the actual error message from a solr response.
         """
         reason = headers.get('reason', None)
         full_html = None
-        
+
         if reason is None:
             reason, full_html = self._scrape_response(headers, response)
-        
+
         msg = "[Reason: %s]" % reason
-        
+
         if reason is None:
-            msg += "\n%s" % full_html
-        
+            msg += "\n%s" % unescape_html(full_html)
+
         return msg
-    
+
     def _scrape_response(self, headers, response):
         """
         Scrape the html response.
@@ -384,32 +449,32 @@ class Solr(object):
         # identify the responding server
         server_type = None
         server_string = headers.get('server', '')
-        
-        if 'jetty' in server_string.lower():
+
+        if server_string and 'jetty' in server_string.lower():
             server_type = 'jetty'
-        
-        if 'coyote' in server_string.lower():
+
+        if server_string and 'coyote' in server_string.lower():
             # TODO: During the pysolr 3 effort, make this no longer a
             #       conditional and consider using ``lxml.html`` instead.
             from BeautifulSoup import BeautifulSoup
             server_type = 'tomcat'
-        
+
         reason = None
         full_html = ''
         dom_tree = None
-        
+
         if server_type == 'tomcat':
             # Tomcat doesn't produce a valid XML response
             soup = BeautifulSoup(response)
             body_node = soup.find('body')
             p_nodes = body_node.findAll('p')
-            
+
             for p_node in p_nodes:
                 children = p_node.findChildren()
-                
+
                 if len(children) >= 2 and 'message' in children[0].renderContents().lower():
                     reason = children[1].renderContents()
-            
+
             if reason is None:
                 full_html = soup.prettify()
         else:
@@ -417,28 +482,30 @@ class Solr(object):
             try:
                 dom_tree = ET.fromstring(response)
                 reason_node = None
-                
+
                 # html page might be different for every server
                 if server_type == 'jetty':
                     reason_node = dom_tree.find('body/pre')
-                
+                else:
+                    reason_node = dom_tree.find('head/title')
+
                 if reason_node is not None:
                     reason = reason_node.text
-                
+
                 if reason is None:
                     full_html = ET.tostring(dom_tree)
             except SyntaxError, e:
                 full_html = "%s" % response
-        
+
         full_html = full_html.replace('\n', '')
         full_html = full_html.replace('\r', '')
         full_html = full_html.replace('<br/>', '')
         full_html = full_html.replace('<br />', '')
         full_html = full_html.strip()
         return reason, full_html
-    
+
     # Conversion #############################################################
-    
+
     def _from_python(self, value):
         """
         Converts python values to a form suitable for insertion into the xml
@@ -459,38 +526,38 @@ class Solr(object):
         else:
             value = unicode(value)
         return value
-    
+
     def _to_python(self, value):
         """
         Converts values from Solr to native Python values.
         """
         if isinstance(value, (int, float, long, complex)):
             return value
-        
+
         if isinstance(value, (list, tuple)):
             value = value[0]
-        
+
         if value == 'true':
             return True
         elif value == 'false':
             return False
-        
+
         if isinstance(value, basestring):
             possible_datetime = DATETIME_REGEX.search(value)
-        
+
             if possible_datetime:
                 date_values = possible_datetime.groupdict()
-            
+
                 for dk, dv in date_values.items():
                     date_values[dk] = int(dv)
-            
+
                 return datetime(date_values['year'], date_values['month'], date_values['day'], date_values['hour'], date_values['minute'], date_values['second'])
-        
+
         try:
             # This is slightly gross but it's hard to tell otherwise what the
             # string's original type might have been. Be careful who you trust.
             converted_value = eval(value)
-            
+
             # Try to handle most built-in types.
             if isinstance(converted_value, (list, tuple, set, dict, int, float, long, complex)):
                 return converted_value
@@ -498,50 +565,61 @@ class Solr(object):
             # If it fails (SyntaxError or its ilk) or we don't trust it,
             # continue on.
             pass
-        
+
         return value
-    
+
     def _is_null_value(self, value):
         """
         Check if a given value is ``null``.
-        
+
         Criteria for this is based on values that shouldn't be included
         in the Solr ``add`` request at all.
         """
         # TODO: This should probably be removed when solved in core Solr level?
         return (value is None) or (isinstance(value, basestring) and len(value) == 0)
-    
+
     # API Methods ############################################################
-    
+
     def search(self, q, **kwargs):
         """Performs a search and returns the results."""
         params = {'q': q}
         params.update(kwargs)
         response = self._select(params)
-        
+
         # TODO: make result retrieval lazy and allow custom result objects
         result = self.decoder.decode(response)
         result_kwargs = {}
-        
+
+        if result.get('debug'):
+            result_kwargs['debug'] = result['debug']
+
         if result.get('highlighting'):
             result_kwargs['highlighting'] = result['highlighting']
-        
+
         if result.get('facet_counts'):
             result_kwargs['facets'] = result['facet_counts']
-        
+
         if result.get('spellcheck'):
             result_kwargs['spellcheck'] = result['spellcheck']
-        
+
         if result.get('stats'):
             result_kwargs['stats'] = result['stats']
-        
-        self.log.debug("Found '%s' search results." % result['response']['numFound'])
-        return Results(result['response']['docs'], result['response']['numFound'], **result_kwargs)
-    
+
+        if 'QTime' in result.get('responseHeader', {}):
+            result_kwargs['qtime'] = result['responseHeader']['QTime']
+
+        if result.get('grouped'):
+            result_kwargs['grouped'] = result['grouped']
+
+        response = result.get('response') or {}
+        numFound = response.get('numFound', 0)
+        self.log.debug("Found '%s' search results.", numFound)
+        return Results(response.get('docs', ()), numFound, **result_kwargs)
+
     def more_like_this(self, q, mltfl, **kwargs):
         """
         Finds and returns results similar to the provided query.
-        
+
         Requires Solr 1.3+.
         """
         params = {
@@ -550,25 +628,25 @@ class Solr(object):
         }
         params.update(kwargs)
         response = self._mlt(params)
-        
+
         result = self.decoder.decode(response)
-        
+
         if result['response'] is None:
             result['response'] = {
                 'docs': [],
                 'numFound': 0,
             }
-        
-        self.log.debug("Found '%s' MLT results." % result['response']['numFound'])
+
+        self.log.debug("Found '%s' MLT results.", result['response']['numFound'])
         return Results(result['response']['docs'], result['response']['numFound'])
-    
+
     def suggest_terms(self, fields, prefix, **kwargs):
         """
         Accepts a list of field names and a prefix
-        
+
         Returns a dictionary keyed on field name containing a list of
         ``(term, count)`` pairs
-        
+
         Requires Solr 1.4+.
         """
         params = {
@@ -580,21 +658,26 @@ class Solr(object):
         result = self.decoder.decode(response)
         terms = result.get("terms", {})
         res = {}
-        
-        while terms:
-            # The raw values are a flat list: ["dance",23,"dancers",10,"dancing",8,"dancer",6]]
-            field = terms.pop(0)
-            values = terms.pop(0)
+
+        # in Solr 1.x the value of terms is a flat list:
+        #   ["field_name", ["dance",23,"dancers",10,"dancing",8,"dancer",6]]
+        #
+        # in Solr 3.x the value of terms is a dict:
+        #   {"field_name": ["dance",23,"dancers",10,"dancing",8,"dancer",6]}
+        if isinstance(terms, types.ListType):
+            terms = dict(zip(terms[0::2], terms[1::2]))
+
+        for field, values in terms.iteritems():
             tmp = list()
-            
+
             while values:
                 tmp.append((values.pop(0), values.pop(0)))
-            
+
             res[field] = tmp
-        
+
         self.log.debug("Found '%d' Term suggestions results.", sum(len(j) for i, j in res.items()))
         return res
-    
+
     def add(self, docs, commit=True, boost=None, commitWithin=None, waitFlush=None, waitSearcher=None):
         """Adds or updates documents. For now, docs is a list of dictionaries
         where each key is the field name and each value is the value to index.
@@ -606,51 +689,51 @@ class Solr(object):
             message.set('commitWithin', commitWithin)
         for doc in docs:
             d = ET.Element('doc')
-            
+
             for key, value in doc.items():
                 if key == 'boost':
                     d.set('boost', str(value))
                     continue
-                
+
                 # handle lists, tuples, and other iterables
                 if hasattr(value, '__iter__'):
                     for v in value:
                         if self._is_null_value(value):
                             continue
-                        
+
                         if boost and v in boost:
                             if not isinstance(boost, basestring):
                                 boost[v] = str(boost[v])
-                            
+
                             f = ET.Element('field', name=key, boost=boost[v])
                         else:
                             f = ET.Element('field', name=key)
-                        
+
                         f.text = self._from_python(v)
                         d.append(f)
                 # handle strings and unicode
                 else:
                     if self._is_null_value(value):
                         continue
-                    
+
                     if boost and key in boost:
                         if not isinstance(boost, basestring):
                             boost[key] = str(boost[key])
-                        
+
                         f = ET.Element('field', name=key, boost=boost[key])
                     else:
                         f = ET.Element('field', name=key)
-                    
+
                     f.text = self._from_python(value)
                     d.append(f)
-            
+
             message.append(d)
-        
+
         m = ET.tostring(message, encoding='utf-8')
         end_time = time.time()
-        self.log.debug("Built add request of %s docs in %0.2f seconds." % (len(docs), end_time - start_time))
+        self.log.debug("Built add request of %s docs in %0.2f seconds.", len(docs), end_time - start_time)
         response = self._update(m, commit=commit, waitFlush=waitFlush, waitSearcher=waitSearcher)
-    
+
     def delete(self, id=None, q=None, commit=True, waitFlush=None, waitSearcher=None):
         """Deletes documents."""
         if id is None and q is None:
@@ -661,16 +744,16 @@ class Solr(object):
             m = '<delete><id>%s</id></delete>' % id
         elif q is not None:
             m = '<delete><query>%s</query></delete>' % q
-        
+
         response = self._update(m, commit=commit, waitFlush=waitFlush, waitSearcher=waitSearcher)
-    
+
     def commit(self, waitFlush=None, waitSearcher=None, expungeDeletes=None):
         if expungeDeletes is not None:
             msg = '<commit expungeDeletes="%s" />' % str(bool(expungeDeletes)).lower()
         else:
             msg = '<commit />'
         response = self._update(msg, waitFlush=waitFlush, waitSearcher=waitSearcher)
-    
+
     def optimize(self, waitFlush=None, waitSearcher=None, maxSegments=None):
         if maxSegments:
             msg = '<commit maxSegments="%d" />' % maxSegments
@@ -678,11 +761,86 @@ class Solr(object):
             msg = '<commit />'
         response = self._update('<optimize />', waitFlush=waitFlush, waitSearcher=waitSearcher)
 
+    def extract(self, file_obj, extractOnly=True, **kwargs):
+        """
+        POSTs a file to the Solr ExtractingRequestHandler so rich content can
+        be processed using Apache Tika. See the Solr wiki for details:
+
+            http://wiki.apache.org/solr/ExtractingRequestHandler
+
+        The ExtractingRequestHandler has a very simply model: it extracts
+        contents and metadata from the uploaded file and inserts it directly
+        into the index. This is rarely useful as it allows no way to store
+        additional data or otherwise customize the record. Instead, by default
+        we'll use the extract-only mode to extract the data without indexing it
+        so the caller has the opportunity to process it as appropriate; call
+        with ``extractOnly=False`` if you want to insert with no additional
+        processing.
+
+        Returns None if metadata cannot be extracted; otherwise returns a
+        dictionary containing at least two keys:
+
+            :contents:
+                        Extracted full-text content, if applicable
+            :metadata:
+                        key:value pairs of text strings
+        """
+        if not POSTER_AVAILABLE:
+            raise RuntimeError("Solr rich content extraction requires `poster` to be installed")
+
+        # The poster library unfortunately defaults to mime-type None when
+        # the file lacks a name and that causes it to send the file contents
+        # as a gigantic string rather than a separate MIME part, which breaks
+        # and spews the contents in the Solr request log:
+        if not hasattr(file_obj, "name"):
+            raise ValueError("extract() requires file-like objects which have a defined name property")
+
+        params = {
+            "extractOnly": "true" if extractOnly else "false",
+            "lowernames": "true",
+            "wt": "json",
+            # We'll provide the file using its true name as Tika may use that
+            # as a file type hint:
+            file_obj.name: file_obj,
+        }
+        params.update(kwargs)
+
+        body_generator, headers = multipart_encode(params)
+
+        try:
+            resp = self._send_request('POST', "%s/update/extract" % self.path,
+                                      "".join(body_generator), headers)
+        except (IOError, SolrError),  e:
+            self.log.error("Failed to extract document metadata: %s", e,
+                           exc_info=True)
+            raise
+
+        try:
+            data = json.loads(resp)
+        except ValueError, e:
+            self.log.error("Failed to load JSON response: %s", e,
+                           exc_info=True)
+            raise
+
+        data['contents'] = data.pop(file_obj.name, None)
+        data['metadata'] = metadata = {}
+
+        raw_metadata = data.pop("%s_metadata" % file_obj.name, None)
+
+        if raw_metadata:
+            # The raw format is somewhat annoying: it's a flat list of
+            # alternating keys and value lists
+            while raw_metadata:
+                metadata[raw_metadata.pop()] = raw_metadata.pop()
+
+        return data
+
+
 
 class SolrCoreAdmin(object):
     """
     Handles core admin operations: see http://wiki.apache.org/solr/CoreAdmin
-    
+
     Operations offered by Solr are:
        1. STATUS
        2. CREATE
@@ -696,25 +854,25 @@ class SolrCoreAdmin(object):
     def __init__(self, url, *args, **kwargs):
         super(SolrCoreAdmin, self).__init__(*args, **kwargs)
         self.url = url
-    
+
     def _get_url(self, url, params={}, headers={}):
         request = urllib2.Request(url, data=safe_urlencode(params), headers=headers)
         # Let ``socket.error``, ``urllib2.HTTPError`` and ``urllib2.URLError``
         # propagate up the stack.
         response = urllib2.urlopen(request)
         return response.read()
-    
+
     def status(self, core=None):
         """http://wiki.apache.org/solr/CoreAdmin#head-9be76f5a459882c5c093a7a1456e98bea7723953"""
         params = {
             'action': 'STATUS',
         }
-        
+
         if core is not None:
             params.update(core=core)
-        
+
         return self._get_url(self.url, params=params)
-    
+
     def create(self, name, instance_dir=None, config='solrcofig.xml', schema='schema.xml'):
         """http://wiki.apache.org/solr/CoreAdmin#head-7ca1b98a9df8b8ca0dcfbfc49940ed5ac98c4a08"""
         params = {
@@ -723,14 +881,14 @@ class SolrCoreAdmin(object):
             'config': config,
             'schema': schema,
         }
-        
+
         if instance_dir is None:
             params.update(instanceDir=name)
         else:
             params.update(instanceDir=instance_dir)
-        
+
         return self._get_url(self.url, params=params)
-    
+
     def reload(self, core):
         """http://wiki.apache.org/solr/CoreAdmin#head-3f125034c6a64611779442539812067b8b430930"""
         params = {
@@ -738,7 +896,7 @@ class SolrCoreAdmin(object):
             'core': core,
         }
         return self._get_url(self.url, params=params)
-    
+
     def rename(self, core, other):
         """http://wiki.apache.org/solr/CoreAdmin#head-9473bee1abed39e8583ba45ef993bebb468e3afe"""
         params = {
@@ -747,11 +905,11 @@ class SolrCoreAdmin(object):
             'other': other,
         }
         return self._get_url(self.url, params=params)
-    
+
     def alias(self, core, other):
         """
         http://wiki.apache.org/solr/CoreAdmin#head-8bf9004eaa4d86af23d2758aafb0d31e2e8fe0d2
-        
+
         Experimental feature in Solr 1.3
         """
         params = {
@@ -760,7 +918,7 @@ class SolrCoreAdmin(object):
             'other': other,
         }
         return self._get_url(self.url, params=params)
-    
+
     def swap(self, core, other):
         """http://wiki.apache.org/solr/CoreAdmin#head-928b872300f1b66748c85cebb12a59bb574e501b"""
         params = {
@@ -769,7 +927,7 @@ class SolrCoreAdmin(object):
             'other': other,
         }
         return self._get_url(self.url, params=params)
-    
+
     def unload(self, core):
         """http://wiki.apache.org/solr/CoreAdmin#head-f5055a885932e2c25096a8856de840b06764d143"""
         params = {
@@ -777,7 +935,7 @@ class SolrCoreAdmin(object):
             'core': core,
         }
         return self._get_url(self.url, params=params)
-        
+
     def load(self, core):
         raise NotImplementedError('Solr 1.4 and below do not support this operation.')
 
@@ -818,10 +976,10 @@ REPLACEMENTS = (
 
 def sanitize(data):
     fixed_string = data
-    
+
     for bad, good in REPLACEMENTS:
         fixed_string = fixed_string.replace(bad, good)
-    
+
     return fixed_string
 
 
